"""
Find roads and assign a zip
Released into the public domain
2010 Andreas Stricker <andy@knitter.ch>
"""
import sys
import osm
import osmreader
from osmconditions import *
import plzreader

def levenshtein_distance(a,b):
    "Calculates the Levenshtein distance between a and b."
    n, m = len(a), len(b)
    if n > m:
        # Make sure n <= m, to use O(min(n,m)) space
        a, b = b, a
        n, m = m, n
        
    current = range(n + 1)
    for i in range(1, m + 1):
        previous, current = current, [i] + [0] * n
        for j in range(1, n + 1):
            add, delete = previous[j] + 1, current[j - 1] + 1
            change = previous[j - 1]
            if a[j - 1] != b[i - 1]:
                change = change + 1
            current[j] = min(add, delete, change)
            
    return current[n]

def readosm(filename):
    reader = osmreader.OsmReader()
    container = reader.parse(filename)
    print container.getStats()
    condition = And(
        IsType('way'),
        HasTag('highway'),
        HasTag('name')
    )
    print "Condition:", repr(condition)
    return container.find(condition)

def readplz(filename):
    return plzreader.readPlzFile(filename)

def match(mapnames, streetlist):
    exact = []
    fuzzy = []
    for plz, name in streetlist:     
        best = None
        bestname = (None, None, None)
        for mapname in mapnames:
            if mapname == name:
                exact.append((name, plz))
                break
            else:
                d = levenshtein_distance(mapname, name)
                if best is None or best > d:
                    best = d
                    bestname = (d, mapname, name, plz)
        else:
            fuzzy.append(bestname)
    return exact, fuzzy

def main():
    streets = readosm(sys.argv[1])
    names = list(set([s.getTag('name') for s in streets]))
    names.sort()
    print "Street segments:", len(streets)
    print "Street names   :", len(names)
    streetlist = readplz(sys.argv[2])
    print "PLZ streets:", len(streetlist)
    exact, fuzzy = match(names, streetlist)
    print "Exact matches:", len(exact)
    for item, plz in exact:
        print "   ", item, "(%s)" % plz
    print "Fuzzy matches:", len(fuzzy)
    fuzzy.sort()
    for d, mapname, name, plz in fuzzy:
        print "    %2d:" % d, mapname, "/", name, "(%s)" % plz 
    return vars()

if __name__ == '__main__':
    main()
