"""
OpenStreetMap Data Primitives search conditions
Released into the public domain
2010 Andreas Stricker <andy@knitter.ch>
"""
import osm

class Condition(object):
    pass

class IsType(Condition):
    def __init__(self, t):
        if isinstance(t, type):
            self._t = t
        else:
            self._t = osm.Container.OSM_PRIMITIVES[t]
    def __repr__(self):
        return "IsType('%s')" % self._t.NAME
    def __call__(self, item):
        return isinstance(item, self._t)

class HasTag(Condition):
    def __init__(self, tag):
        self._tag = tag
    def __repr__(self):
        return "HasTag('%s')" % self._tag
    def __call__(self, item):
        try:
            return item.getTag(self._tag) is not None
        except KeyError:
            return False

class TagEqual(Condition):
    def __init__(self, key, value):
        self._key = key
        self._value = value
    def __repr__(self):
        return "TagEqual('%s', '%s')" % (self._key, self._value)
    def __call__(self, item):
        try:
            return item.getTag(self._key) == self._value
        except KeyError:
            return False

class IsUser(Condition):
    def __init__(self, user):
        self._user = user
    def __repr__(self):
        return "IsUser('%s')" % self._user
    def __call__(self, item):
        return item.getAttribute('user') == self._user

class IsUid(Condition):
    def __init__(self, user):
        self._uid = uid
    def __repr__(self):
        return "IsUid('%s')" % self._uid
    def __call__(self, item):
        return item.getAttribute('uid') == self._uid

class And(Condition):
    def __init__(self, *conditions):
        self._conditions = conditions
    def __repr__(self):
        return "And(%s)" % ", ".join([repr(a) for a in self._conditions])
    def __call__(self, item):
        for cond in self._conditions:
            if not cond(item):
                return False
        return True

class Or(Condition):
    def __init__(self, *conditions):
        self._conditions = conditions
    def __repr__(self):
        return "Or(%s)" % ", ".join([repr(a) for a in self._conditions])
    def __call__(self, item):
        for cond in self._conditions:
            if cond(item):
                return True

class Not(Condition):
    def __init__(self, a):
        self._a = a
    def __repr__(self):
        return "Not(%s)" % repr(self._a)
    def __call__(self):
        return not self._a(item)

